LKML Archive on lore.kernel.org
help / color / mirror / Atom feed
From: Mariusz Bialonczyk <manio@skyboo.net>
To: linux-kernel@vger.kernel.org, Greg Kroah-Hartman <greg@kroah.com>
Cc: Mariusz Bialonczyk <manio@skyboo.net>
Subject: [PATCH 2/4] w1: ds2413: add retry support to state_read()
Date: Mon, 20 May 2019 09:05:56 +0200 [thread overview]
Message-ID: <20190520070558.20142-2-manio@skyboo.net> (raw)
In-Reply-To: <20190520070558.20142-1-manio@skyboo.net>
The state_read() was calling PIO_ACCESS_READ once and bail out if it
failed for this first time.
This commit is improving this to trying more times before it give up,
similarly as the write call is currently doing.
Signed-off-by: Mariusz Bialonczyk <manio@skyboo.net>
---
drivers/w1/slaves/w1_ds2413.c | 37 +++++++++++++++++++++++------------
1 file changed, 24 insertions(+), 13 deletions(-)
diff --git a/drivers/w1/slaves/w1_ds2413.c b/drivers/w1/slaves/w1_ds2413.c
index cd3763df69ac..d63778c70568 100644
--- a/drivers/w1/slaves/w1_ds2413.c
+++ b/drivers/w1/slaves/w1_ds2413.c
@@ -30,6 +30,9 @@ static ssize_t state_read(struct file *filp, struct kobject *kobj,
size_t count)
{
struct w1_slave *sl = kobj_to_w1_slave(kobj);
+ unsigned int retries = W1_F3A_RETRIES;
+ ssize_t bytes_read = -EIO;
+
dev_dbg(&sl->dev,
"Reading %s kobj: %p, off: %0#10x, count: %zu, buff addr: %p",
bin_attr->attr.name, kobj, (unsigned int)off, count, buf);
@@ -42,22 +45,30 @@ static ssize_t state_read(struct file *filp, struct kobject *kobj,
mutex_lock(&sl->master->bus_mutex);
dev_dbg(&sl->dev, "mutex locked");
- if (w1_reset_select_slave(sl)) {
- mutex_unlock(&sl->master->bus_mutex);
- return -EIO;
- }
+ if (w1_reset_select_slave(sl))
+ goto out;
- w1_write_8(sl->master, W1_F3A_FUNC_PIO_ACCESS_READ);
- *buf = w1_read_8(sl->master);
+ while (retries--) {
+ w1_write_8(sl->master, W1_F3A_FUNC_PIO_ACCESS_READ);
- mutex_unlock(&sl->master->bus_mutex);
- dev_dbg(&sl->dev, "mutex unlocked");
+ *buf = w1_read_8(sl->master);
+ /* check for correct complement */
+ if ((*buf & 0x0F) == ((~*buf >> 4) & 0x0F)) {
+ bytes_read = 1;
+ goto out;
+ }
- /* check for correct complement */
- if ((*buf & 0x0F) != ((~*buf >> 4) & 0x0F))
- return -EIO;
- else
- return 1;
+ if (w1_reset_resume_command(sl->master))
+ goto out; /* unrecoverable error */
+
+ dev_warn(&sl->dev, "PIO_ACCESS_READ error, retries left: %d\n", retries);
+ }
+
+out:
+ mutex_unlock(&sl->master->bus_mutex);
+ dev_dbg(&sl->dev, "%s, mutex unlocked, retries: %d\n",
+ (bytes_read > 0) ? "succeeded" : "error", retries);
+ return bytes_read;
}
static BIN_ATTR_RO(state, 1);
--
2.19.0.rc1
next prev parent reply other threads:[~2019-05-20 7:06 UTC|newest]
Thread overview: 8+ messages / expand[flat|nested] mbox.gz Atom feed top
2019-05-20 7:05 [PATCH 1/4] w1: ds2413: output_write() cosmetic fixes / simplify Mariusz Bialonczyk
2019-05-20 7:05 ` Mariusz Bialonczyk [this message]
2019-05-20 7:05 ` [PATCH 3/4] w1: ds2413: when the slave is not responding during read, select it again Mariusz Bialonczyk
2019-05-22 10:40 ` [PATCH 3/4 v2] " Mariusz Bialonczyk
2019-05-20 7:05 ` [PATCH 4/4] w1: ds2805: rename w1_family struct, fixing c-p typo Mariusz Bialonczyk
2019-05-24 18:21 ` Greg KH
2019-05-25 8:45 ` [PATCH 4/4 v2] " Mariusz Bialonczyk
2019-05-25 8:51 ` [PATCH 4/4] " Mariusz Bialonczyk
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20190520070558.20142-2-manio@skyboo.net \
--to=manio@skyboo.net \
--cc=greg@kroah.com \
--cc=linux-kernel@vger.kernel.org \
--subject='Re: [PATCH 2/4] w1: ds2413: add retry support to state_read()' \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).